/*

128Coding.H - Main 128Gamma Encryption Algorithm

extern int IncodeMain(char *Input_File,char *Output_File)
extern int DecodeMain(char *Input_File, char *Output_File)

void ClearCodeVariables(void);
void AssembleByte(void);
void IncodeByte(void);

*/

/////////////////////////////////////////////////////////////////////

struct _CodeByte {
unsigned short A, B, C, D, E;
unsigned short F, G, H5[5], I, J;
unsigned short K, L, M, N;
}; struct _CodeByte Byte;

unsigned char Key[75];
short SingleChar;

FILE *Source, *Target;

void ClearCodeVariables(void);
void AssembleByte(void);
void IncodeByte(void);

//--------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////

extern int IncodeMain(char *Input_File,char *Output_File)
{
	Byte.E = 0;
	Byte.G = 0;
	Byte.J = 0;
	strcpy(Key,CodedKey);

	DWORD SourceFileSize = 0;
	long double Current_Size = 0;

	SourceFileSize = FileSize(Input_File);
	if(SourceFileSize == -1)
	{
		CancelFlag = TRUE;
		return 0;
	}


	if ((Source = fopen(Input_File,"rb")) == NULL)
	{
		if(Source)fclose(Source);
		Error("Could Not Open Source File For Incode. Error: 3.");
		ClearCodeVariables();
		return 0;
	}

	if ((Target = fopen(Output_File,"wb")) == NULL)
	{
		if(Source)fclose(Source);
		if(Target)fclose(Target);
		Error("Could Not Open Target File For Incode. Error: 4.");
		ClearCodeVariables();
		return 0;
	}

	/* Write Flags To File */

	if(CompressEncrypt)fprintf(Target,ENCCompress);
	if(CompressPack)fprintf(Target,PACCompress);

	if(Pack)fprintf(Target,PACFlag);
	if(Encrypt)fprintf(Target,ENCFlag);

	while(( SingleChar = fgetc(Source)) != EOF)
	{
		Current_Size++;
		StatusValue = ((Current_Size / SourceFileSize) * 100);
		if(CancelFlag)
		{
			if(Source)fclose(Source);
			if(Target)fclose(Target);
			return 0;
		}

		AssembleByte();
		Byte.L = Byte.K >> 8;
		Byte.M = Byte.K &255;
		for (Byte.N = 0; Byte.N <= 9; Byte.N++)
		{
			Key[Byte.N] = Key[Byte.N] ^ SingleChar;
		}
		SingleChar = SingleChar ^ (Byte.L^Byte.M);

		fputc(SingleChar,Target);
	}
	if(Source)fclose(Source);
	if(Target)fclose(Target);
	ClearCodeVariables();
	return 1;
}

//////////////////////////////////////////////////////////////////////

extern int DecodeMain(char *Input_File, char *Output_File)
{
	Byte.E = 0;
	Byte.G = 0;
	Byte.J = 0;
	strcpy(Key,CodedKey);

	DWORD SourceFileSize = 0;
	long double Current_Size = 0;

	SourceFileSize = FileSize(Input_File);
	if(SourceFileSize == -1)
	{
		CancelFlag = TRUE;
		return 0;
	}

	if ((Source = fopen(Input_File,"rb")) == NULL)
	{
		if(Source)fclose(Source);
		Error("Could Not Open Source File For Decode. Error: 6.");
		ClearCodeVariables();
		return 0;
	}

	fseek(Source,FlagLEN,SEEK_SET); // Seek Past Flag
	Current_Size += 10; 			// Compensate Status For Seek

	if ((Target = fopen(Output_File,"wb")) == NULL)
	{
		if(Source)fclose(Source);
		if(Target)fclose(Target);
		Error("Could Not Open Target File For Decode. Error: 7.");
		ClearCodeVariables();
		return 0;
	}

	while ( (SingleChar = fgetc(Source)) != EOF)
	{
		Current_Size++;
		StatusValue = ((Current_Size / SourceFileSize) * 100);
		if(CancelFlag)
		{
			if(Source)fclose(Source);
			if(Target)fclose(Target);
			CleanUp(0,1);
			return 0;
		}

		AssembleByte();
		Byte.L = Byte.K >> 8;
		Byte.M = Byte.K &255;
		SingleChar = SingleChar ^ (Byte.L ^ Byte.M);
		for ( Byte.N = 0; Byte.N <= 9; Byte.N++ )
		{
			Key[Byte.N] = Key[Byte.N]^SingleChar;
		}

		fputc(SingleChar,Target);
	}
	if(Source)fclose(Source);
	if(Target)fclose(Target);
	ClearCodeVariables();
	return 1;
}

//////////////////////////////////////////////////////////////////////

void ClearCodeVariables(void)
{
	int LocalLoop = 0;
	for(LocalLoop = 0; LocalLoop < sizeof(Key); LocalLoop++)
	{
		Key[Byte.N] = 0;
	}

	Byte.A = 0;
	Byte.B = 0;
	Byte.C = 0;
	Byte.D = 0;
	Byte.E = 0;
	Byte.F = 0;
	Byte.G = 0;
	Byte.H5[0] = 0;
	Byte.H5[1] = 0;
	Byte.H5[2] = 0;
	Byte.H5[3]=0;
	Byte.H5[4] = 0;
	Byte.I = 0;
	Byte.J = 0;
	Byte.K = 0;
	Byte.L = 0;
	Byte.M = 0;
	Byte.N = 0;
	SingleChar = 0;
}

//////////////////////////////////////////////////////////////////////

void AssembleByte(void)
{
	Byte.H5[0] = ( Key[0] * 256 ) + Key[1];
	IncodeByte();
	Byte.K = Byte.I;
	Byte.H5[1] = Byte.H5[0] ^ ( (Key[2] * 256) + Key[3] );
	IncodeByte();
	Byte.K = Byte.K ^ Byte.I;
	Byte.H5[2] = Byte.H5[1] ^ ( (Key[4] * 256) + Key[5] );
	IncodeByte();
	Byte.K = Byte.K ^ Byte.I;
	Byte.H5[3] = Byte.H5[2] ^ ( (Key[6] * 256) + Key[7] );
	IncodeByte();
	Byte.K = Byte.K ^ Byte.I;
	Byte.H5[4] = Byte.H5[3] ^ ( (Key[8] * 256) + Key[9] );
	IncodeByte();
	Byte.K = Byte.K ^ Byte.I;
	Byte.J = 0;
}

//////////////////////////////////////////////////////////////////////

void IncodeByte(void)
{
	Byte.D = Byte.G + Byte.J;
	Byte.A = Byte.H5[Byte.J];
	Byte.C = 0x015a;
	Byte.B = 0x4e35;
	Byte.F = Byte.A;
	Byte.A = Byte.E;
	Byte.E = Byte.F;
	Byte.F = Byte.A;
	Byte.A = Byte.D;
	Byte.D = Byte.F;
	if (Byte.A != 0)
	{
		Byte.A = Byte.A * Byte.B;
	}
	Byte.F = Byte.A;
	Byte.A = Byte.C;
	Byte.C = Byte.F;
	if (Byte.A != 0)
	{
		Byte.A = Byte.A * Byte.E;
		Byte.C = Byte.A + Byte.C;
	}
	Byte.F = Byte.A;
	Byte.A = Byte.E;
	Byte.E = Byte.F;
	Byte.A = Byte.A * Byte.B;
	Byte.D = Byte.C + Byte.D;
	Byte.A = Byte.A + 1;
	Byte.G = Byte.D;
	Byte.H5[Byte.J] = Byte.A;
	Byte.I = Byte.A ^ Byte.D;
	Byte.J = Byte.J + 1;
}

//////////////////////////////////////////////////////////////////////

